import axios from 'axios'
import url from '@/plugins/routes'
const timeout = 20000

const state = {
  free_drink_list_config: [],
  free_drink_list_history: [],
  free_drink_list_reasons: [],
  free_drink_types: [],
  free_drink_list_config_loading: false,
  free_drink_list_history_loading: false,
  free_drink_list_reasons_loading: false,
  free_drink_types_loading: false,
  snackbar_messages: []
}

const mutations = {
  set_free_drink_list_config(state, data) {
    state.free_drink_list_config = data
  },
  set_free_drink_list_history(state, data) {
    data.forEach(item => {
      item.timestamp = new Date(
        item.timestamp.year,
        item.timestamp.month - 1,
        item.timestamp.day,
        item.timestamp.hour,
        item.timestamp.minute,
        item.timestamp.second
      )
    })
    state.free_drink_list_history = data
    state.free_drink_list_history.reverse()
  },
  set_free_drink_list_config_loading(state, loading) {
    state.free_drink_list_config_loading = loading
  },
  set_free_drink_list_history_loading(state, loading) {
    state.free_drink_list_history_loading = loading
  },
  set_free_drink_list_reasons_loading(state, loading) {
    state.free_drink_list_reasons_loading = loading
  },
  add_snackbar_message(state, { data, error }) {
    data.error = error
    data.visible = true
    data.free_drink_type = data.free_drink_types.find(item => {
      return item.id == data.free_drink_type_id
    })
    data.timer = setTimeout(() => {
      if (!data.error) {
        data.visible = false
      }
    }, 5000)
    console.log(data)
    state.snackbar_messages.unshift(data)
  },
  set_free_drink_list_reasons(state, data) {
    state.free_drink_list_reasons = data
  },
  set_free_drink_types(state, data) {
    state.free_drink_types = data
  },
  set_free_drink_types_loading(state, loading) {
    state.free_drink_types_loading = loading
  },
  set_free_drink_config(state, data) {
    let config = state.free_drink_list_config.find(item => {
      return item.id == data.id
    })
    Object.assign(config, data)
  }
}

const actions = {
  async get_free_drink_list_config({ commit, dispatch, rootState }) {
    try {
      commit('set_free_drink_list_config_loading', true)
      const response = await axios.get(url.freeDrinkListConfig, {
        headers: { Token: rootState.login.user.accessToken },
        timeout
      })
      commit('set_free_drink_list_config', response.data)
    } catch (e) {
      if (e.message == 'Network Error') {
        dispatch('connectionError/addError', null, { root: true })
      }
      if (e.response)
        if (e.response.status === 401) dispatch('logout', null, { root: true })
    } finally {
      commit('set_free_drink_list_config_loading', false)
    }
  },
  async get_free_drink_list_history({ commit, dispatch, rootState }) {
    try {
      commit('set_free_drink_list_history_loading', true)
      const response = await axios.get(url.freeDrinkListHistory, {
        headers: { Token: rootState.login.user.accessToken },
        timeout
      })
      commit('set_free_drink_list_history', response.data)
    } catch (e) {
      if (e.message == 'Network Error') {
        dispatch('connectionError/addError', null, { root: true })
      }
      if (e.response)
        if (e.response.status === 401) dispatch('logout', null, { root: true })
    } finally {
      commit('set_free_drink_list_history_loading', false)
    }
  },
  async get_free_drink_list_history_from_to({ commit, dispatch, rootState },  data) {
    try {
      commit('set_free_drink_list_history_loading', true)
      const response = await axios.post(url.freeDrinkListHistoryFromTo, {...data},{
        headers: { Token: rootState.login.user.accessToken },
        timeout
      })
      commit('set_free_drink_list_history', response.data)
    } catch (e) {
      if (e.message == 'Network Error') {
        dispatch('connectionError/addError', null, { root: true })
      }
      if (e.response)
        if (e.response.status === 401) dispatch('logout', null, { root: true })
    } finally {
      commit('set_free_drink_list_history_loading', false)
    }
  },
  async set_free_drink_list_history({ commit, dispatch, rootState }, data) {
    try {
      commit('set_free_drink_list_history_loading', true)
      const response = await axios.post(
        url.freeDrinkListHistory,
        { ...data },
        {
          headers: { Token: rootState.login.user.accessToken },
          timeout
        }
      )
      commit('set_free_drink_list_history', response.data)
      commit('add_snackbar_message', { data, error: false })
    } catch (e) {
      commit('add_snackbar_message', { data, error: true })
      if (e.message == 'Network Error') {
        dispatch('connectionError/addError', null, { root: true })
      }
      if (e.response)
        if (e.response.status === 401) dispatch('logout', null, { root: true })
    } finally {
      commit('set_free_drink_list_history_loading', false)
    }
  },
  async update_free_drink_list_history({ commit, dispatch, rootState }, data) {
    try {
      commit('set_free_drink_list_history_loading', true)
      const response = await axios.put(
        url.freeDrinkListHistory,
        { ...data },
        { headers: { Token: rootState.login.user.accessToken }, timeout }
      )
      commit('set_free_drink_list_history', response.data)
      commit('add_snackbar_message', { data, error: false })
    } catch (e) {
      dispatch('connectionError/addError', null, { root: true })
      if (e.message == 'Network Error') {
        dispatch('connectionError/addError', null, { root: true })
      }
      if (e.response)
        if (e.response.status === 401) dispatch('logout', null, { root: true })
    } finally {
      commit('set_free_drink_list_history_loading', false)
    }
  },
  async get_free_drink_list_reasons({ commit, dispatch, rootState }) {
    try {
      commit('set_free_drink_list_reasons_loading', true)
      const response = await axios.get(url.freeDrinkListReasons, {
        headers: { Token: rootState.login.user.accessToken },
        timeout
      })
      commit('set_free_drink_list_reasons', response.data)
    } catch (e) {
      dispatch('connectionError/addError', null, { root: true })
      if (e.message == 'Network Error') {
        dispatch('connectionError/addError', null, { root: true })
      }
      if (e.response)
        if (e.response.status === 401) dispatch('logout', null, { root: true })
    } finally {
      commit('set_free_drink_list_reasons_loading', false)
    }
  },
  async get_free_drink_types({ commit, dispatch, rootState }) {
    try {
      commit('set_free_drink_types_loading', true)
      const response = await axios.get(url.freeDrinkTypes, {
        headers: { Token: rootState.login.user.accessToken },
        timeout
      })
      commit('set_free_drink_types', response.data)
    } catch (e) {
      dispatch('connectionError/addError', null, { root: true })
      if (e.message == 'Network Error') {
        dispatch('connectionError/addError', null, { root: true })
      }
      if (e.response)
        if (e.response.status === 401) dispatch('logout', null, { root: true })
    } finally {
      commit('set_free_drink_types_loading', false)
    }
  },
  async set_free_drink_list_config({commit, dispatch, rootState}, data) {
    try {
      commit('set_free_drink_list_config_loading', true)
      const response = await axios.post(url.freeDrinkListConfig, {...data},{
        headers: { Token: rootState.login.user.accessToken },
        timeout
      })
      commit('set_free_drink_list_config', response.data)
    } catch (e) {
      dispatch('connectionError/addError', null, { root: true })
      if (e.message == 'Network Error') {
        dispatch('connectionError/addError', null, { root: true })
      }
      if (e.response)
        if (e.response.status === 401) dispatch('logout', null, { root: true })
    } finally {
      commit('set_free_drink_list_config_loading', false)
    }
  },
  async update_free_drink_list_config({commit, dispatch, rootState}, data) {
    try {
      commit('set_free_drink_list_config_loading', true)
      const response = await axios.put(url.freeDrinkListConfig, {...data},{
        headers: { Token: rootState.login.user.accessToken },
        timeout
      })
      commit('set_free_drink_list_config', response.data)
    } catch (e) {
      dispatch('connectionError/addError', null, { root: true })
      if (e.message == 'Network Error') {
        dispatch('connectionError/addError', null, { root: true })
      }
      if (e.response)
        if (e.response.status === 401) dispatch('logout', null, { root: true })
    } finally {
      commit('set_free_drink_list_config_loading', false)
    }
  },
  async delete_free_drink_list_config({commit, dispatch, rootState}, data) {
    try {
      commit('set_free_drink_list_config_loading', true)
      const response = await axios.post(url.deleteFreeDrinkListConfig, {...data},{
        headers: { Token: rootState.login.user.accessToken },
        timeout
      })
      commit('set_free_drink_list_config', response.data)
    } catch (e) {
      dispatch('connectionError/addError', null, { root: true })
      if (e.message == 'Network Error') {
        dispatch('connectionError/addError', null, { root: true })
      }
      if (e.response)
        if (e.response.status === 401) dispatch('logout', null, { root: true })
    } finally {
      commit('set_free_drink_list_config_loading', false)
    }
  },
  async set_free_drink_list_reason({ commit, dispatch, rootState }, data) {
    try {
      commit('set_free_drink_list_reasons_loading', true)
      const response = await axios.post(url.freeDrinkListReasons, {...data},{
        headers: { Token: rootState.login.user.accessToken },
        timeout
      })
      commit('set_free_drink_list_reasons', response.data)
    } catch (e) {
      dispatch('connectionError/addError', null, { root: true })
      if (e.message == 'Network Error') {
        dispatch('connectionError/addError', null, { root: true })
      }
      if (e.response)
        if (e.response.status === 401) dispatch('logout', null, { root: true })
    } finally {
      commit('set_free_drink_list_reasons_loading', false)
    }
  }
  ,
  async update_free_drink_list_reason({ commit, dispatch, rootState }, data) {
    try {
      commit('set_free_drink_list_reasons_loading', true)
      const response = await axios.put(url.freeDrinkListReasons, {...data},{
        headers: { Token: rootState.login.user.accessToken },
        timeout
      })
      commit('set_free_drink_list_reasons', response.data)
    } catch (e) {
      dispatch('connectionError/addError', null, { root: true })
      if (e.message == 'Network Error') {
        dispatch('connectionError/addError', null, { root: true })
      }
      if (e.response)
        if (e.response.status === 401) dispatch('logout', null, { root: true })
    } finally {
      commit('set_free_drink_list_reasons_loading', false)
    }
  },
  async delete_free_drink_list_reason({ commit, dispatch, rootState }, data) {
    try {
      commit('set_free_drink_list_reasons_loading', true)
      const response = await axios.post(url.deleteFreeDrinkListReason, {...data},{
        headers: { Token: rootState.login.user.accessToken },
        timeout
      })
      commit('set_free_drink_list_reasons', response.data)
    } catch (e) {
      dispatch('connectionError/addError', null, { root: true })
      if (e.message == 'Network Error') {
        dispatch('connectionError/addError', null, { root: true })
      }
      if (e.response)
        if (e.response.status === 401) dispatch('logout', null, { root: true })
    } finally {
      commit('set_free_drink_list_reasons_loading', false)
    }
  }

}

const getters = {
  free_drink_list_config(state) {
    return state.free_drink_list_config
  },
  free_drink_list_config_job(state) {
    return state.free_drink_list_config.filter(item => {
      console.log(item)
      return item.free_drink_types.find(drink_type => {
        return drink_type.id == 1
      })
    })
  },
  free_drink_list_config_workgroup(state) {
    return state.free_drink_list_config.filter(item => {
      return item.free_drink_types.find(drink_type => {
        return drink_type.id == 2
      })
    })
  },
  free_drink_list_config_band(state) {
    return state.free_drink_list_config.filter(item => {
      return item.free_drink_types.find(drink_type => {
        return drink_type.id == 3
      })
    })
  },
  free_drink_list_history(state) {
    return state.free_drink_list_history
  },
  free_drink_list_history_without_canceled(state) {
    return state.free_drink_list_history.filter(item => {
      return !item.canceled
    })
  },
  free_drink_list_history_band(state) {
    return state.free_drink_list_history.filter(item => {
      return item.free_drink_type.id == 3
    })
  },
  free_drink_list_history_band_without_canceled(state) {
    return state.free_drink_list_history.filter(item => {
      return item.free_drink_type.id == 3 && !item.canceled
    })
  },
  free_drink_list_history_job_without_canceled(state) {
    return state.free_drink_list_history.filter(item => {
      return item.free_drink_type.id == 1 && !item.canceled
    })
  },
  free_drink_list_history_job_credit(state) {
    let jobDrinks = state.free_drink_list_history.filter(item => {
      return item.free_drink_type.id == 1 && !item.canceled
    })
    let credit = 0
    jobDrinks.forEach(item => {
      credit += item.free_drink_config.price
    })
    return credit
  },
  free_drink_list_history_workgroup_without_canceled(state) {
    return state.free_drink_list_history.filter(item => {
      return item.free_drink_type.id == 2 && !item.canceled
    })
  },
  free_drink_list_history_workgroup(state) {
    return state.free_drink_list_history.filter(item => {
      return item.free_drink_type.id == 2
    })
  },
  loading(state) {
    return (
      state.free_drink_list_history_loading ||
      state.free_drink_list_config_loading
    )
  },
  snackbar_messages(state) {
    return state.snackbar_messages
  },
  free_drink_list_reasons(state) {
    return state.free_drink_list_reasons
  },
  free_drink_list_reasons_loading(state) {
    return state.free_drink_list_reasons_loading
  },
  free_drink_types(state) {
    return state.free_drink_types
  },
  free_drink_list_history_loading(state) {
    return state.free_drink_list_history_loading
  },
  free_drink_list_config_loading(state) {
    return state.free_drink_list_config_loading
  },
  free_drink_types_loading(state) {
    return state.free_drink_types_loading
  },
  free_drink_list_config_by_id: (state) => (id) => {
    return state.free_drink_list_config.find(item => { return item.id == id})
  }
}

export default {
  namespaced: true,
  state,
  mutations,
  actions,
  getters
}
